# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class Deprecated_PassphraseStateRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 77

    def __init__(
        self,
        _state: bytes = None,
    ) -> None:
        self._state = _state

    @classmethod
    def get_fields(cls):
        return {
            1: ('_state', p.BytesType, 0),
        }
